#!/usr/bin/env bash
# Build script for Scriptarious

set -o errexit
set -o nounset
set -o pipefail
if [[ "${TRACE-0}" == "1" ]]; then
    set -o xtrace
fi

RED="\033[0;31m"
GREEN="\033[0;33m"
YELLOW="\033[1;33m"
PURPLE="\033[0;35m"
NORMAL="\033[0m"

BOLDBLUE="\033[1;94m"

ERRORSTR="${RED}ERROR${NORMAL}"

usage()
{
	echo "Usage: build REQUIRED [OTHERS]"
	echo -e "\tREQUIRED"
	echo -e "\t-c <channel>   \tUpdate chanel (beta or stable)."
        echo -e "\t-t <build_type>\tBuild type (devel or prod, prod is the default)."
        echo -e "\t-r             \tGenerate release notes in appcast.xml."
	echo ""
	echo -e "\tOTHERS"
	echo -e "\t-h             \tShow this help"
}

#
# Some defines
#

APPLICATION_NAME="Scriptarious"
YAPPLICATION_NAME="${YELLOW}${APPLICATION_NAME}${NORMAL}"

ROOTDIR="."
WORKSPACE=
PROJECT="${ROOTDIR}/Scriptarious/${APPLICATION_NAME}.xcodeproj"
TARGET="${APPLICATION_NAME}"
SCHEME="${APPLICATION_NAME} (Production)"
EXPORTPLIST="${ROOTDIR}/Scripts/exportPList.plist"

WEBSITEURL="https://scriptarious.moxadventu.com"
DOWNLOAD_PREFIX="${WEBSITEURL}/update"

RELEASE_NOTES_URL_PREFIX="${WEBSITEURL}"

# Build type devel or prod
BUILDTYPE="prod"

# Chanel is stable or beta
CHANNEL=""

# Release notes
ADD_RELEASE_NOTES=""

#
# Start of script
#

echo -e "Build application: ${YAPPLICATION_NAME}"

#
# Arguments parsing
#

OPTS=`getopt hc:t:r $*`

if [ $? -ne 0 ]; then
  usage
  exit 1
fi

set -- $OPTS

while true; do
	case "$1" in
		-h)
			usage
			exit 0
			;;
		-c)
			CHANNEL="$2"
			shift 2
			;;
		-t)
			BUILDTYPE="$2"
			shift 2
			;;
   -r)
    ADD_RELEASE_NOTES="YES"
    shift
    ;;
		--)
			shift
			break 
			;;
	esac
done

#
# Check parameters
#

if [ -z "$CHANNEL" ]; then
	echo -e "${ERRORSTR}: Update channel is not specified. Should be 'beta' or 'stable'."
	exit 1
fi
if [ "$CHANNEL" != "beta" ] && [ "$CHANNEL" != "stable" ]; then
	echo -e "${ERRORSTR}: Bad update chanel. Should be 'beta' or 'stable'."
	usage
	exit 1
fi

if [ ! -z "$BUILDTYPE" ]; then
	case "$BUILDTYPE" in
		devel)
			SCHEME="${APPLICATION_NAME} (Development)"
			;;
		prod)
			# nothing to change
			;;
		*)
			echo -e "${ERRORSTR} '$BUILDTYPE' is not a valid build type"
			usage
			exit 1
			;;
	esac
fi

YSCHEME="${YELLOW}${SCHEME}${NORMAL}"

#
# Archive & export application
#

ARCHIVENAME=${APPLICATION_NAME}-${CHANNEL}.xcarchive
UPDATEDIR="${ROOTDIR}/Updates/"
ARCHIVEPATH="${UPDATEDIR}/Archives/${ARCHIVENAME}"

echo -e "Building ${YAPPLICATION_NAME} with scheme ${YSCHEME}"
if [ ! -z "$ADD_RELEASE_NOTES" ]; then
  echo -e "Generation release notes"
fi
echo -e "Output in ${BOLDBLUE}${UPDATEDIR}${NORMAL}"

read -p"(Press [ENTER] to continue or ^C to stop)"

# Create missing folders
[ ! -d "${UPDATEDIR}" ] && mkdir "${UPDATEDIR}"

# Build
xcodebuild archive -quiet -project "${PROJECT}" -scheme "${SCHEME}" -archivePath "${ARCHIVEPATH}" -destination "generic/platform=macOS"
xcodebuild -quiet -exportArchive -archivePath "${ARCHIVEPATH}" -exportPath "${UPDATEDIR}" -exportOptionsPlist "${EXPORTPLIST}" -destination "generic/platform=macOS" -allowProvisioningUpdates

#
# Notarize application
#

ZIPNAME="${APPLICATION_NAME}.zip"
SOURCENAME="${UPDATEDIR}/${APPLICATION_NAME}.app"
DESTINATIONNAME="${UPDATEDIR}/${ZIPNAME}"

echo -e "Notarize ${YELLOW}${APPLICATION_NAME}${NORMAL}..."

ditto -c -k --keepParent ${SOURCENAME} ${DESTINATIONNAME}

xcrun notarytool submit "${DESTINATIONNAME}" \
                 --keychain-profile "MoxAdventuNotarization" \
                 --wait -v

echo -e "Stable ${YAPPLICATION_NAME}"

xcrun stapler staple "${SOURCENAME}"

#
# Preparing update
#

echo -e "Preparing Sparkle update ${YAPPLICATION_NAME}..."

INFOPLIST_PATH="${SOURCENAME}/Contents/Info.plist"
VERSIONSTRING=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "${INFOPLIST_PATH}")
BUNDLEVERSION=$(/usr/libexec/PlistBuddy -c "Print :CFBundleVersion" "${INFOPLIST_PATH}")

FULLVERSIONNAME="${APPLICATION_NAME}-${BUILDTYPE}-${CHANNEL}-${VERSIONSTRING}-(${BUNDLEVERSION})"

SPARKLEUPDATEFOLDER="${UPDATEDIR}/${FULLVERSIONNAME}"
FINALZIPNAME="${FULLVERSIONNAME}.zip"

[ ! -d "${SPARKLEUPDATEFOLDER}" ] && mkdir "${SPARKLEUPDATEFOLDER}"
rm -f "${SPARKLEUPDATEFOLDER}/${FINALZIPNAME}"
ditto -c -k --keepParent "${SOURCENAME}" "${SPARKLEUPDATEFOLDER}/${FINALZIPNAME}"

GENAPPCAST_OPTIONS="--download-url-prefix \"${DOWNLOAD_PREFIX}\""
GENAPPCAST_OPTIONS="${GENAPPCAST_OPTIONS} --link ${WEBSITEURL}"
GENAPPCAST_OPTIONS="${GENAPPCAST_OPTIONS} --maximum-deltas 0"

if [ "${CHANNEL}" = "beta" ]; then
	GENAPPCAST_OPTIONS="${GENAPPCAST_OPTIONS} --channel beta"
fi

# Add release notes URL
if [ ! -z "${ADD_RELEASE_NOTES}" ]; then
  
  # Release notes files
  RELEASE_NOTES_MARKDOWN="./ReleaseNotes/${VERSIONSTRING}(${BUNDLEVERSION}).md"
  RELEASE_NOTES_HTML="${SPARKLEUPDATEFOLDER}/${FULLVERSIONNAME}.html"

  if [[ ! -f "${RELEASE_NOTES_MARKDOWN}" ]]; then
    echo -e "Release note file ${RELEASE_NOTES_MARKDOWN} doesnt exists."
    exit 1
  fi

  # Convert markdown file to HTML
  markdown "${RELEASE_NOTES_MARKDOWN}" > $RELEASE_NOTES_HTML

#  GENAPPCAST_OPTIONS="${GENAPPCAST_OPTIONS} --release-notes-url-prefix ${RELEASE_NOTES_URL_PREFIX}"
fi

echo -e "Generate appcast for ${YAPPLICATION_NAME} with options: ${GENAPPCAST_OPTIONS}"

"${ROOTDIR}/Tools/Sparkle/bin/generate_appcast" ${GENAPPCAST_OPTIONS} "${SPARKLEUPDATEFOLDER}"

#
# Generate ZIP for base download
#

ditto -c -k --keepParent "${SOURCENAME}" "${SPARKLEUPDATEFOLDER}/${ZIPNAME}"

#
# Keep dSYMs files
#

ditto -c -k --keepParent -rsrc "${ARCHIVEPATH}/dSYMs" ${SPARKLEUPDATEFOLDER}/${FULLVERSIONNAME}-dSYM.zip

echo -e "${GREEN}Done${NORMAL}"
open .
